#
#
# This code uses the random walk Metropolis-Hastings algorithm with 
# diffuse priors to estimate the model proposed by Goto, Jacobs, 
# Sinclair, and van Norden in their paper 'Employment Reconciliation 
# and Nowcasting' published in the Journal of Applied Econometrics.
#
#------------------------------------------------------------------------------#
#     
#                              Preparation
#
#------------------------------------------------------------------------------#
rm(list = ls())   # Remove all the objects from the workspace
getwd()           # Current working directory 
setwd('')    #Please set your directory
#External packages
library("readxl");library("FKF");
#Functions
source("ssmodel.R"); source("fastloglike.R"); source("fastllf.R")
source("kalmancal.R"); source("fastkalman.R")

#------------------------------------------------------------------------------#
#                                Set up
#------------------------------------------------------------------------------#
model_type  <- 3 ## PLEASE LEAVE AS IS
# 1: No const no slope, 2: Include const no slope, 3: Include const and slope (benchmark)
if (model_type == 1){numpara <- 15
}else if(model_type == 2){numpara <- 16
}else if(model_type == 3){numpara <- 17; 
}
c <- 0.003              # Scaling factor (a tuning parameter), targeting 20-30% acceptance rate
reps <- 50000           # Number of simulation use 50,000
burn <- 0.1*reps        # Number of burn-in
#------------------------------------------------------------------------------
#                                 Data
#------------------------------------------------------------------------------
EMPData<- as.matrix(read_xlsx("Employment Data.xlsx", sheet = 1)) # Import the data
dimEMPData <- dim(EMPData)      # The dimension of the data
Tsample_x<- dimEMPData[1]       # The sample size
z1 <- as.numeric(EMPData[,2])   # Household data
x1 <- as.numeric(EMPData[,3])   # Payroll data (first release)
x2 <- as.numeric(EMPData[,4])   # Payroll data (second release)
x3 <- as.numeric(EMPData[,5])   # Payroll data (third release)
l = 3                           # Number of first l releases

#------------------------------------------------------------------------------#
#                              Measurement vector (X)
#------------------------------------------------------------------------------#
X <- cbind(z1, x1, x2, x3)     # The measurement vector

#------------------------------------------------------------------------------#
#                              Initial State Vector 
#------------------------------------------------------------------------------#
initial_alpha<-as.vector(rep(0,l+2))    # The initial state vector that contains all zeros. 
dim_alpha<-length(initial_alpha)        # The dimension of the state vector
P_00 <- diag(1E6, dim_alpha,dim_alpha)  # The initial state covariance

#------------------------------------------------------------------------------#
#                              
#                  Bayesian Estimation - Diffuse prior 
#          Use maximum likelihood estimates as initial parameters
# 
#------------------------------------------------------------------------------#

MLE_para<- as.matrix(read_xlsx("MLE_est.xlsx", sheet = 1))   # Import the initial prameters
MLE_cov<- as.matrix(read_xlsx("MLE_est.xlsx", sheet = 2))    # Import the initial covariance matrix

gamma_old <- as.numeric(MLE_para[1:numpara,3])    # Initial parameter
cov0<- MLE_cov[1:numpara, 32:(31+numpara)]        # Covariance matrix 
H0<-diag(abs(diag(cov0)));                        # Uncertainty matrix for drawing
H0[2,2]<-H0[9,9]<-(sd(z1))^2                      # Adjust the uncertainty for the HH data
P<-chol(H0*c)                                     
# Cholsky decomposition of the product of the uncertainty matrix and the scaling factor

ten_p<-reps/10

output1 <- matrix(NA, (reps-burn), numpara) # Store parameters 

arpara <- gamma_old[1]                      # Initial AR parameter
sigpara <- gamma_old[2:15]                  # Initial standard error parameters 
if (model_type == 2){
  constpara <- gamma_old[16]                # Initial constant parameter
}else if (model_type ==3){
  constpara <- gamma_old[16]                # Initial constant parameter
  slopepara <- gamma_old[17]                # Initial slope parameter
}

#------------------------------------------------------------------------------#
#                              Prior (diffuse)
#  Para 1, AR parameter, Rho ~ N(0.5, 0.2)
#  Para 2-5,9-12. Standard deviations, FD: sigma ~ U(0,1000) 
#  Para 6-8,13-15. Standard deviations, FD:sigma ~ U(-2000,2000) 
#  Para 16, constant, mu ~ N(mu, 100), mu comes from mean(PR3) - mean(HH)
#  Para 17, slope, beta ~ N(1, 0.2)
#------------------------------------------------------------------------------#

armu0 <- 0.5; arV0 <- 0.2                                     # for para 1

sigmu0a <- c(rep(0,4),rep(-1000,3),rep(0,4),rep(-1000,3));    # for para 2-5, 9-12
sigmu0b <- c(rep(2000,4),rep(1000,3),rep(2000,4),rep(1000,3)) # for para 6-8, 13-15

MLE_const <- mean(X[,4], na.rm=TRUE) - mean(X[,1], na.rm=TRUE)
if (!model_type == 1){
  constmu0 <-MLE_const; constV0 <- 100                        # for para 16
}
if (model_type == 3){
  slopemu0 <- 1; slopeV0 <- 0.2                               # for para 17
}
#------------------------------------------------------------------------------#
#                           Evaluate the log prior
#------------------------------------------------------------------------------#
# para 1
arprior <- -log(arV0) - log(2*pi)/2 -( t(arpara - armu0)%*%solve(arV0)%*%(arpara - armu0)) # Log prior from AR parameter
# paras 2-15
sigprior <- 0;
for (i in 1:14){
  sigprior <- sigprior + log( 1/(sigmu0b[i]-sigmu0a[i]) )   # Log prior from standard deviation parameter
} 
# para 16
if (!model_type == 1){
  constprior <- -log(constV0) - log(2*pi)/2 -( t(constpara - constmu0)%*%solve(constV0)%*%(constpara - constmu0)) # Log prior from constant parameter
}
# para 17
if (model_type == 3){
  slopeprior <- -log(slopeV0) - log(2*pi)/2 -( t(slopepara - slopemu0)%*%solve(slopeV0)%*%(slopepara - slopemu0)) # Log prior from beta parameter
}
# log prior total
if (model_type ==1){
  prior_old <- arprior + sigprior 
} else if (model_type == 2){
  prior_old <- arprior + sigprior +constprior
} else if (model_type == 3){
  prior_old <- arprior + sigprior +constprior + slopeprior
}   

#------------------------------------------------------------------------------#
#                            Evaluate log posterior
#------------------------------------------------------------------------------#
# Calculate log likelihood
like<-fastllf(gamma_old,X,initial_alpha, P_00,l, model_type) 
# Calculate log posterior
posterior_old <- like + prior_old    

#------------------------------------------------------------------------------#
#
#                     Random walk Metropolis-Hastings
# 
#------------------------------------------------------------------------------#
set.seed(06222022)               
num_accept<-0
output2<- rep(NA, reps-burn)
jj <- 1;

for (i in 1:reps){ # MCMC simulation
  
  gamma_new <- gamma_old + rnorm(numpara, 0, 1)%*%P  # New parameters draw
  
  check <- all(gamma_new[2:5]>=0 & gamma_new[9:12]>=0)  # Test if the constraints are met
  if (isTRUE(check)){
    like <-fastllf(gamma_new, X,initial_alpha, P_00, l, model_type)     # new log likelihood
    arpara_new <- gamma_new[1]                                          # New AR parameter
    sigpara_new <- gamma_new[2:15]                                      # New standard error parameters
    arprior_new <- -log(arV0) - log(2*pi)/2 - ( t(arpara_new - armu0)%*%solve(arV0)%*%(arpara_new - armu0))       # New log prior from AR parameter
    sigprior_new <- sigprior                                            # because uniform dist
    if (!model_type == 1){
      constpara_new <- gamma_new[16]                                    # New constant parameter
      constprior_new <- -log(constV0) - log(2*pi)/2 - ( t(constpara_new - constmu0)%*%solve(constV0)%*%(constpara_new - constmu0))       # New log prior from constant parameter
    }
    if (model_type == 3){
      slopepara_new <- gamma_new[17]                                    # New slope parameter
      slopeprior_new <- -log(slopeV0) - log(2*pi)/2 - ( t(slopepara_new - slopemu0)%*%solve(slopeV0)%*%(slopepara_new - slopemu0))       # New log prior from slope parameter
    }
    if (model_type ==1){
      prior_new <- arprior_new + sigprior_new 
    } else if (model_type == 2){
      prior_new <- arprior_new + sigprior_new +constprior_new
    } else if (model_type == 3){
      prior_new <- arprior_new + sigprior_new +constprior_new + slopeprior_new
    }   
    posterior_new = like + prior_new                                    # New log posterior
  } else {
    posterior_new = -10^100 # If the constraints are not met, impose very small new log posterior. 
  }
  
  accept <- min(exp(posterior_new - posterior_old), 1)   # Acceptance probability
  u<-runif(1, 0, 1)    # A random draw from uniform dist with [0,1]
  
  if (u < accept){ # If accept is greater than the uniform distribution draw
    gamma_old<-gamma_new             # update the paramter estimates      
    posterior_old<-posterior_new     # update the posterior
    num_accept <-num_accept + 1      # update the # accepted
  }
  accept_rate = num_accept / i;
  if (i%%ten_p==0){
    print(round(100*accept_rate))
    print(" % acceptance rate ")
    print(i/ten_p*10)
    print(" % done")
    flush.console()
  }
  if (i>burn){
    output1[jj,]<-gamma_old          # Record the parameters 
    output2[jj]<-posterior_old       # Record the posterior
    jj<-jj+1
  }
} 
thetas <- output1                    # All the parameters 

sorted_theta <- apply(thetas, 2, sort, decreasing=F)
theta_med<-sorted_theta[round(0.5*(reps-burn)),]  # Median parameter 
theta_25<-sorted_theta[round(0.25*(reps-burn)),]  # 25th% parameter 
theta_75<-sorted_theta[round(0.75*(reps-burn)),]  # 75th% parameter 
#------------------------------------------------------------------------------#
#                   Obtain the state/latent variables
#------------------------------------------------------------------------------#
# Obtain filtered variables 
kalman_est_mcmc<-fastkalman(theta_med, X,initial_alpha, P_00,l,model_type)
# Obtain filtered latent employment 
latent_mcmc<- kalman_est_mcmc$att[1,]
# Obtain smoothed variables 
kalmansmo_est_mcmc<-fks(kalman_est_mcmc)
# Obtain smoothed latent employment 
latent_smo_mcmc<-kalmansmo_est_mcmc$ahatt[1,]  

# Plot 1st release payroll and smoothed latent
plot(ts(x1,start = c(1961,2), frequency = 12),col = 2,ylab = "Employment")
lines(ts(latent_smo_mcmc,start = c(1961,2), frequency = 12),col = 5)
legend("bottomright",c("Payroll employment (1st release)", "Latent (smoothed) series"),lty = 1,col = c(2,5))


#-----------------------------------------------------------------------------
#                   Calculate the Kalman Gains
#------------------------------------------------------------------------------
# Construct lagged edge data
Yragged <- X
Ynobs <- nrow(Yragged)
Yragged[Ynobs,3:4] <- NA
Yragged[Ynobs-1,4] <- NA

# Obtain filtered variables 
kalman_est_mcmc <-
  fastkalman(
    theta_med,
    Yragged,
    initial_alpha = initial_alpha,
    P_00 = P_00,
    l = l,
    model_type = model_type
    )

# Obtain Kalman gains
KGains_mcmc <- na.omit(as.vector(kalman_est_mcmc$Kt[1,,(Ynobs-2):Ynobs]))
KGains_mcmc <- data.frame(KGains_mcmc,row.names = c("H[T-2]", "PR1[T-2]", "PR2[T-2]",
                                                    "PR3[T-2]", "H[T-1]", "PR1[T-1]",
                                                    "PR2[T-1]", "H[T]", "PR1[T]"))

#Display results
mcmc_estimate <- cbind(theta_25,theta_med,theta_75) # binds all the vectors together
colnames(mcmc_estimate) <- c('25%','Median','75%')
if (model_type==1) {rownames(mcmc_estimate) <- c('rho', 's.nu.H', 's.nu.1', 's.nu.2', 's.nu.3', 's.nu.H1', 
                                                 's.nu.H2','s.nu.H3', 's.eps.H', 's.eps.1', 's.eps.2', 's.eps.3',
                                                 's.eps.H1', 's.eps.H2', 's.eps.H3')
}else if (model_type==2){rownames(mcmc_estimate) <- c('rho', 's.nu.H', 's.nu.1', 's.nu.2', 's.nu.3', 's.nu.H1', 
                                                      's.nu.H2','s.nu.H3', 's.eps.H', 's.eps.1', 's.eps.2', 's.eps.3',
                                                      's.eps.H1', 's.eps.H2', 's.eps.H3', 'constant')
}else if (model_type==3) {rownames(mcmc_estimate) <- c('rho', 's.nu.H', 's.nu.1', 's.nu.2', 's.nu.3', 's.nu.H1', 
                                                       's.nu.H2','s.nu.H3', 's.eps.H', 's.eps.1', 's.eps.2', 's.eps.3',
                                                       's.eps.H1', 's.eps.H2', 's.eps.H3', 'constant', 'slope')}
mcmc_estimate
print(format(KGains_mcmc, digits = 2))
kalman_est_mcmc$logLik

